# Yii2 Flysystem REST API

## Installation

The preferred way to install this extension is through [composer](http://getcomposer.org/download/).

```bash
composer require eluhr/yii2-flysystem-rest-api
```

## Setup

Add the module to your web application configuration:

```php
<?php
use eluhr\flysystemRestApi\components\FileStorage;
use eluhr\flysystemRestApi\Module as FileSystemRestModule;
use League\Flysystem\Local\LocalFilesystemAdapter;
use yii\web\JsonParser;

return [
    'modules' => [
        'filesystem-rest' => [
            'class' => FileSystemRestModule::class
        ]
    ],
    'components' => [
        'fileStorage' => [
            'class' => FileStorage::class,
            'adapter' => new LocalFilesystemAdapter(
                '/mnt/storage'
            )
        ],  
        'urlManager' => [
            // This is only needed if your using codemix/yii2-localeurls (https://github.com/codemix/yii2-localeurls)
            'ignoreLanguageUrlPatterns' => [
                '#^filesystem-rest/api#' => '#^filesystem-rest/api#'
            ]
        ],
        'request' => [
            'parsers' => [
                'application/json' => JsonParser::class
            ]
        ],
    ]
];
```

And this to your console application configuration:

```php
[
    'controllerMap' => [
        'migrate' => [
            'migrationPath' => [
                '@vendor/eluhr/yii2-flysystem-rest-api/src/migrations',
                // '@vendor/eluhr/yii2-flysystem-rest-api/src/migrations/plugins/permissions', // If you want to use the permission plugin
                // '@vendor/eluhr/yii2-flysystem-rest-api/src/migrations/plugins/category', // If you want to use the permission plugin
                // '@vendor/eluhr/yii2-flysystem-rest-api/src/migrations/plugins/alias', // If you want to use the permission plugin
            ]
        ]
    ]
]
```

or run

```bash
yii migrate/up --migrationPath=@vendor/dmstr/yii2-oauth-module/src/migrations
```


## TBD

 - Meta table for info
 - Meta table for permission
 - Implement component
 - Implement cli
 - Implement API
 - Tests
 - Pagination? (Load more)

## Plugins

 - Rename
 - Permissions

## Examples

### Catch exceptions

To catch exceptions thrown by the file storage component you can use the `EVENT_EXCEPTION` event. The event `data` contains the exception.
Configure your component like this:

```php
use eluhr\flysystemRestApi\components\FileStorage;
use eluhr\flysystemRestApi\events\FileStorageErrorEvent;

return [
    'components' => [
        'fileStorage' => [
            'class' => FileStorage::class,
            'on ' . FileStorageEvent::EVENT_EXCEPTION => function (FileStorageEvent $event) {
                Yii::error($event->data->getMessage());
            }
        ]
    ]
];
```

### Setup jwt authentication

See https://github.com/bizley/yii2-jwt/blob/master/INSTRUCTION.md
