function createFilemanager (id, options) {
    new Vue({
        el: '#' + id,
        created () {
            this.listContent()
        },
        data () {

            const _options = {
                allowUpload: true,
                allowCreateDirectory: true,
                showBreadcrumb: true,
                showSearch: true,
                showCategoryFilter: true,
                urls: {
                    list: null,
                    uploadFiles: null,
                    createDirectory: null,
                    deleteDirectory: null,
                    deleteFile: null
                },
                plugins: {
                    category: {
                        urls: {
                            updateCategory: null
                        }
                    },
                    alias: {
                        urls: {
                            updateAlias: null
                        }
                    }
                },
                messages: {
                    deleteItem: 'Are you sure you want to delete this item?'
                }
            }

            return {
                isLoading: false, // Loading state for e.g. list files
                options: merge(options, _options),
                path: '',
                items: [],
                model: {
                    files: [],
                    directoryName: '',
                    searchTerm: '',
                    selectedCategory: ''
                },
                contextMenu: {
                    visible: false,
                    item: null,
                    position: {
                        top: 0,
                        left: 0
                    }
                },
                plugins: {
                    category: {
                        categories: [
                            {id: 1, name: 'Nachweise'},
                            {id: 2, name: 'Deminimis-Bescheide'},
                            {id: 3, name: 'Zertifizierung'},
                            {id: 4, name: 'Vollmachten'},
                            {id: 5, name: 'Sonstiges'},
                        ]
                    },
                    alias: {
                        editUuid: null,
                        newName: null
                    }
                },
                sortAttribute: 'lastModified',
                sortDirection: 'desc'
            }
        },
        computed: {
            paths () {
                var paths = this.path.split('/')
                return paths.map(function (elem, index) {
                    return paths.slice(0, index + 1).reduce(function (a, b) {
                        return {
                            name: b,
                            path: (a.path || a) + '/' + b
                        }
                    })
                }).filter(Boolean)
            },
            contextMenuStyles () {
                return {
                    position: 'absolute',
                    top: this.contextMenu.position.top + 'px',
                    left: this.contextMenu.position.left + 'px'
                }
            },
            filteredItems () {

                const searchInAlias = this.isPluginEnabled('alias')
                const searchTerm = this.model.searchTerm.toLowerCase()

                const filterByCategory = this.isPluginEnabled('category')
                const selectedCategory = this.model.selectedCategory
                const filteredItems = this.items.filter(item => {

                    // Find by name or alias
                    let name;
                    if (searchInAlias && item.extraMetadata.alias.alias) {
                        name = item.extraMetadata.alias.alias
                    } else {
                        name = item.name
                    }
                    name = name.toLowerCase()
                    const foundByName = name.indexOf(searchTerm) > -1

                    // Find by category
                    let foundByCategory
                    if (filterByCategory && selectedCategory) {
                        foundByCategory = item.extraMetadata.category === selectedCategory
                    } else {
                        foundByCategory = true
                    }


                    return foundByName && foundByCategory
                })

                if (this.sortDirection && this.sortAttribute) {

                    return filteredItems.sort((a,b) => {

                        if (this.isPluginEnabled('category') && this.sortAttribute === 'category') {
                            if(this.sortDirection === 'desc') {
                                return -1
                            }
                            return 1
                        }

                        if (this.isPluginEnabled('alias') && this.sortAttribute === 'name') {
                            let aName, bName;
                            if (a.extraMetadata.alias.alias) {
                                aName = a.extraMetadata.alias.alias
                            } else {
                                aName = a.name
                            }
                            if (b.extraMetadata.alias.alias) {
                                bName = b.extraMetadata.alias.alias
                            } else {
                                bName = b.name
                            }

                            if (this.sortDirection === 'asc') {
                                return aName > bName ? 1 : -1
                            } else {
                                return aName > bName ? -1 : 1
                            }
                        }

                        if (this.isPluginEnabled('extension') && this.sortAttribute === 'extension') {
                            const aExtension = a.extraMetadata.extension.extension
                            const bExtension = b.extraMetadata.extension.extension

                            if (this.sortDirection === 'asc') {
                                return aExtension > bExtension ? 1 : -1
                            } else {
                                return aExtension > bExtension ? -1 : 1
                            }
                        }

                        let modifier = 1;
                        if(this.sortDirection === 'desc') {
                            modifier = -1
                        }
                        if(a[this.sortAttribute] < b[this.sortAttribute]){
                            return -1 * modifier
                        }
                        if(a[this.sortAttribute] > b[this.sortAttribute]) {
                            return 1
                        }
                        return 0;
                    })
                }
                return filteredItems
            }
        },
        methods: {
            listContent () {
                this.isLoading = true
                const url = this.options.urls.list + this.path
                fetch(url)
                    .then(response => {
                        if (response.ok) {
                            return response.json()
                        }
                        throw Error(response.statusText)
                    })
                    .then(data => {
                        this.items = data
                    })
                    .finally(() => this.isLoading = false)
            },
            uploadFilesInternal () {
                this.isLoading = true
                var data = new FormData()
                data.append('basePath', this.path)
                for (var i = 0; i < this.model.files.length; i++) {
                    data.append('files[]', this.model.files[i])
                }
                const url = this.options.urls.uploadFiles
                return fetch(url, {
                    method: 'POST',
                    body: data
                })
                    .then(function (response) {
                        if (response.ok) {
                            return response.json()
                        }
                        throw Error(response.statusText)
                    })
                    .then(() => {
                        this.files = []
                        this.listContent()
                    })

            },
            uploadFiles (event) {
                this.model.files = event.target.files || []
                if (this.model.files.length > 0) {
                    this.uploadFilesInternal().finally(() => {
                        this.isLoading = false
                        event.target.value = null // Reset file input
                    })
                }
            },
            createDirectory () {
                if (this.model.directoryName) {
                    this.isLoading = true
                    const url = this.options.urls.createDirectory
                    fetch(url, {
                        method: 'POST',
                        headers: {
                            'Content-Type': 'application/json'
                        },
                        body: JSON.stringify({
                            basePath: this.path,
                            name: this.model.directoryName
                        })
                    })
                        .then(function (response) {
                            if (response.ok) {
                                return response.json()
                            }
                            throw Error(response.statusText)
                        })
                        .then(data => {
                            this.model.directoryName = ''
                            if (data.success) {
                                this.listContent()
                            }
                        })
                        .finally(() => this.isLoading = false)
                }
            },
            deleteItem (item) {
                if (!confirm(this.options.messages.deleteItem)) {
                    return;
                }
                this.isLoading = true
                let url
                if (item.type === 'dir') {
                    url = this.options.urls.deleteDirectory
                } else {
                    url = this.options.urls.deleteFile
                }
                url += item.uuid
                fetch(url, {
                    method: 'DELETE'
                })
                    .then(function (response) {
                        if (response.ok) {
                            return response.json()
                        }
                        throw Error(response.statusText)
                    })
                    .then(data => {
                        if (data.success) {
                            this.listContent()
                        }
                    })
                    .finally(() => this.isLoading = false)

            },
            moveToDirectory (path) {
                this.path = path
                this.listContent()
            },
            fullPathFromItem (item) {
                return [item.path, item.name].filter(Boolean).join('/')
            },
            openDirectory (item) {
                if (item.type === 'dir') {
                    // Build new path. Filter with boolean will remove all empty values
                    this.moveToDirectory(this.fullPathFromItem(item))
                }
            },
            openContextMenu (item, event) {
                this.contextMenu.visible = true
                this.contextMenu.item = item

                var target = document.getElementById(id)
                var rect = target.getBoundingClientRect()
                var x = event.clientX - rect.left
                var y = event.clientY - rect.top
                this.contextMenu.position.left = x + 10
                this.contextMenu.position.top = y + 10
            },
            closeContextMenu () {
                if (this.contextMenu.visible) {
                    this.contextMenu.visible = false
                    this.contextMenu.item = null
                    this.contextMenu.position.left = 0
                    this.contextMenu.position.top = 0
                }
            },
            isPluginEnabled (pluginName) {
                if (!this.items[0]) {
                    return false
                }
                return typeof this.items[0].extraMetadata[pluginName] !== 'undefined'
            },
            openFilePicker () {
                this.$refs.fileUploadInput.click()
            },
            uploadFileDragDrop (event) {
                // https://developer.mozilla.org/en-US/docs/Web/API/HTML_Drag_and_Drop_API/File_drag_and_drop#process_the_drop
                if (event.dataTransfer.items) {
                    let files = [];
                    // Use DataTransferItemList interface to access the file(s)
                    [...event.dataTransfer.items].forEach((item, i) => {
                        // If dropped items aren't files, reject them
                        if (item.kind === 'file') {
                            files.push(item.getAsFile())
                        }
                    })
                    if (files.length > 0) {
                        this.model.files = files
                        this.uploadFilesInternal().finally(() => {
                            this.isLoading = false
                        })
                    }
                }
            },
            dragOverFile (event) {
                // https://stackoverflow.com/questions/6756583/prevent-browser-from-loading-a-drag-and-dropped-file#comment-51502784
                event.preventDefault()
                document.querySelector('#' + id + ' .items').classList.add('drag-n-drop-upload-overlay')
            },
            dragLeaveFile () {
                document.querySelector('#' + id + ' .items').classList.remove('drag-n-drop-upload-overlay')
            },
            navigationButtonIsDisabled (pathObj) {
                let _path
                if (typeof pathObj === 'string') {
                    _path = pathObj
                } else {
                    _path = pathObj.path
                }

                return _path === this.path
            },
            changeCategory (item, event) {
                const selectedValue = event.target.value
                const category = this.plugins.category.categories.find(cat => Number(cat.id) === Number(selectedValue))
                const newItem = this.items.find(_item => _item === item)

                this.isLoading = true
                const url = this.options.plugins.category.urls.updateCategory
                fetch(url, {
                    method: 'PUT',
                    headers: {
                        'Content-Type': 'application/json'
                    },
                    body: JSON.stringify({
                        itemId: item.uuid,
                        categoryName: category.name
                    })
                })
                    .then(function (response) {
                        if (response.ok) {
                            return response.json()
                        }
                        throw Error(response.statusText)
                    })
                    .then(data => {
                        if (data.success) {
                            this.listContent()
                            newItem.extraMetadata.category = category.name
                        } else {
                            throw Error(data.message)
                        }
                    })
                    .finally(() => {
                        this.isLoading = false
                    })
            },
            itemLabel (item) {
                if (this.isPluginEnabled('alias')) {
                    return item.extraMetadata.alias.alias || item.extraMetadata.alias.original
                }
                return item.name
            },
            startRenameItem (item) {
                if (this.plugins.alias.editUuid === item.uuid) {
                    this.plugins.alias.editUuid = null
                    this.plugins.alias.newName = null
                } else {
                    this.plugins.alias.editUuid = item.uuid
                    this.plugins.alias.newName = item.name
                }
            },
            renameItem (item) {
                this.isLoading = true
                const url = this.options.plugins.alias.urls.updateAlias
                fetch(url, {
                    method: 'PUT',
                    headers: {
                        'Content-Type': 'application/json'
                    },
                    body: JSON.stringify({
                        itemId: item.uuid,
                        aliasName: this.plugins.alias.newName
                    })
                })
                    .then(function (response) {
                        if (response.ok) {
                            return response.json()
                        }
                        throw Error(response.statusText)
                    })
                    .then(data => {
                        if (data.success) {
                            this.listContent()
                        } else {
                            throw Error(data.message)
                        }
                    })
                    .finally(() => {
                        this.isLoading = false
                        this.plugins.alias.editUuid = null
                        this.plugins.alias.newName = null
                    })
            },
            sort (attribute) {
                if(attribute === this.sortAttribute) {
                    this.sortDirection = this.sortDirection === 'asc' ? 'desc' : 'asc';
                }
                this.sortAttribute = attribute;
            }
        },
        filters: {
            timestampToDate (timestamp) {
                if (!timestamp) {
                    return '-'
                }
                return new Intl.DateTimeFormat(navigator.language, {dateStyle: 'medium'}).format(new Date(timestamp * 1000))
            },
            prettyBytes (bytes) {
                if (!bytes) {
                    return '-'
                }
                const decimals = 2

                // https://stackoverflow.com/questions/15900485/correct-way-to-convert-size-in-bytes-to-kb-mb-gb-in-javascript#answer-18650828
                const k = 1000
                const dm = decimals < 0 ? 0 : decimals
                const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB']

                const i = Math.floor(Math.log(bytes) / Math.log(k))

                return `${parseFloat((bytes / Math.pow(k, i)).toFixed(dm))} ${sizes[i]}`
            },
            uppercase (string) {
                return string.toUpperCase()
            }
        }
    })
}

// https://gist.github.com/ahtcx/0cd94e62691f539160b32ecda18af3d6?permalink_comment_id=3889214#gistcomment-3889214
function merge (source, target) {
    for (const [key, val] of Object.entries(source)) {
        if (val !== null && typeof val === `object`) {
            if (target[key] === undefined) {
                target[key] = new val.__proto__.constructor()
            }
            merge(val, target[key])
        } else {
            target[key] = val
        }
    }
    return target
}
