<?php

namespace eluhr\flysystemRestApi\controllers\api;

use eluhr\flysystemRestApi\controllers\BaseController;
use eluhr\flysystemRestApi\models\plugins\StorageItemAlias;
use eluhr\flysystemRestApi\models\plugins\StorageItemCategory;
use Yii;
use yii\base\InvalidConfigException;

class PluginsController extends BaseController
{
    /**
     * @inheritdoc
     */
    protected function verbs()
    {
        $verbs = parent::verbs();
        $verbs['update-category'] = ['PUT', 'PATCH'];
        $verbs['update-alias'] = ['PUT', 'PATCH'];
        return $verbs;
    }

    /**
     * Updated the category of a given item
     *
     * @throws InvalidConfigException
     * @return string[]
     */
    public function actionUpdateCategory(): array
    {
        $itemId = $this->request->post('itemId');
        $categoryName = $this->request->post('categoryName');

        $categoryModel = StorageItemCategory::findOne(['storage_item_id' => $itemId]);
        if ($categoryModel === null) {
            $categoryModel = Yii::createObject(StorageItemCategory::class);
            $categoryModel->setAttribute('storage_item_id', $itemId);
        }
        $categoryModel->setAttribute('name', $categoryName);

        return [
            'success' => $categoryModel->save(),
            'message' => array_values($categoryModel->getFirstErrors())[0] ?? ''
        ];
    }

    /**
     * Updated the alias of a given item
     *
     * @throws InvalidConfigException
     * @return string[]
     */
    public function actionUpdateAlias(): array
    {
        $itemId = $this->request->post('itemId');
        $aliasName = $this->request->post('aliasName');

        $aliasModel = StorageItemAlias::findOne(['storage_item_id' => $itemId]);
        if ($aliasModel === null) {
            $aliasModel = Yii::createObject(StorageItemAlias::class);
            $aliasModel->setAttribute('storage_item_id', $itemId);
        }
        $aliasModel->setAttribute('name', $aliasName);

        return [
            'success' => $aliasModel->save(),
            'message' => array_values($aliasModel->getFirstErrors())[0] ?? ''
        ];
    }
}
