<?php

namespace eluhr\flysystemRestApi\helpers;


use eluhr\flysystemRestApi\models\StorageItem;
use League\Flysystem\StorageAttributes;
use Ramsey\Uuid\Uuid;
use Yii;

class StorageItemHelper
{
    /**
     * Generate uuid for this item based on path, name and type
     *
     * @param string $type
     * @param string $path
     * @param string $name
     *
     * @return string
     */
    public static function generateUuid(string $type, string $path, string $name): string
    {
        return Uuid::uuid5(Uuid::NAMESPACE_DNS, $type . $path . $name)->toString();
    }


    /**
     * @param string $path
     * @param string $name
     *
     * @throws \yii\base\InvalidConfigException
     * @return bool
     */
    public static function upsertFileInDb(string $path, string $name): bool
    {
        return static::upsertItemInDb(StorageAttributes::TYPE_FILE, $path, $name);
    }

    /**
     * @param string $path
     * @param string $name
     *
     * @throws \yii\base\InvalidConfigException
     * @return bool
     */
    public static function upsertDirectoryInDb(string $path, string $name): bool
    {
        return static::upsertItemInDb(StorageAttributes::TYPE_DIRECTORY, $path, $name);
    }



    /**
     * Create a new storage item by the given parameters
     *
     * @param string $type
     * @param string $path
     * @param string $name
     *
     * @throws \yii\base\InvalidConfigException
     * @return bool
     */
    public static function createItemInDb(string $type, string $path, string $name): bool
    {
        $model = Yii::createObject(StorageItem::class);
        $model->setAttributes([
            'type' => $type,
            'path' => $path,
            'name' => $name
        ]);

        return $model->save();
    }

    /**
     * @param string $type
     * @param string $path
     * @param string $name
     *
     * @throws \yii\base\InvalidConfigException
     * @return bool
     */
    public static function upsertItemInDb(string $type, string $path, string $name): bool
    {
        $model = StorageItem::findOne(['id' => StorageItemHelper::generateUuid($type, $path, $name)]);

        if ($model === null) {
            $model = Yii::createObject(StorageItem::class);
        }

        $model->setAttributes([
            'type' => $type,
            'path' => $path,
            'name' => $name
        ]);

        return $model->save();
    }
}
