<?php

namespace eluhr\flysystemRestApi\models;

use eluhr\flysystemRestApi\helpers\StorageItemHelper;
use yii\base\InvalidConfigException;

class Directory extends FileStorageModel
{

    public const SCENARIO_CREATE = 'create';

    /**
     * @var string
     */
    public $basePath;

    /**
     * @var string
     */
    public $name;

    /**
     * @inheritdoc
     */
    public function scenarios(): array
    {
        $scenarios = parent::scenarios();
        $scenarios[self::SCENARIO_CREATE] = [
            'basePath',
            'name'
        ];
        return $scenarios;
    }

    /**
     * @inheritdoc
     */
    public function rules(): array
    {
        $rules = parent::rules();
        $rules[] = [
            'name',
            'required',
            'on' => self::SCENARIO_CREATE
        ];
        $rules[] = [
            [
                'basePath',
                'name'
            ],
            'string',
            'on' => self::SCENARIO_CREATE
        ];
        return $rules;
    }

    /**
     * Build path from base path and name
     *
     * @return string
     */
    public function getPath(): string
    {
        return static::buildPath($this->basePath, $this->name);
    }

    public static function buildPath(string $basePath,string $name): string
    {
        return implode(DIRECTORY_SEPARATOR, array_filter([$basePath, $name]));
    }


    /**
     * Create a new directory after validating
     *
     * @throws InvalidConfigException
     * @return bool
     */
    public function create(): bool
    {
        if ($this->validate()) {
            // Create recursive directories
            $parts = array_filter(explode(DIRECTORY_SEPARATOR, $this->name));
            $newPath = $this->basePath;
            $length = count($parts);
            for ($i = 0; $i < $length; $i++) {
                $nameParts = [];
                // resolve base path
                for ($j = 0; $j < $i + 1; $j++) {
                    $nameParts[] = $parts[$j];
                }
                $name = implode(DIRECTORY_SEPARATOR, $nameParts);
                $path = static::buildPath($this->basePath, $name);

                if ($this->getFileStorage()->createDirectory($path) && StorageItemHelper::upsertDirectoryInDb($newPath, $parts[$i])) {
                    $newPath = implode(DIRECTORY_SEPARATOR, [$newPath, $parts[$i]]);
                } else {
                    return false;
                }
            }
            return true;
        }
        return false;
    }

    /**
     * @inheritdoc
     */
    public function getResponse(): array
    {
        $path = $this->getPath();

        $action = false;

        if ($this->getScenario() === self::SCENARIO_CREATE) {
            $action = $this->create();
        }

        return [
            'success' => $action,
            'path' => $path,
            'message' => array_values($this->getFirstErrors())[0] ?? ''
        ];
    }
}
