<?php

namespace eluhr\flysystemRestApi\models;

use eluhr\flysystemRestApi\helpers\StorageItemHelper;
use yii\base\InvalidConfigException;
use yii\helpers\Inflector;
use yii\web\UploadedFile;

class File extends FileStorageModel
{

    public const SCENARIO_UPLOAD = 'upload';

    /**
     * @var string
     */
    public $basePath;

    /**
     * @var UploadedFile[]
     */
    public $files;

    /**
     * @var array
     */
    private $_infos = [];

    /**
     * @inheritdoc
     */
    public function scenarios(): array
    {
        $scenarios = parent::scenarios();
        $scenarios[self::SCENARIO_UPLOAD] = [
            'basePath',
            'files'
        ];
        return $scenarios;
    }

    /**
     * @inheritdoc
     */
    public function rules(): array
    {
        $rules = parent::rules();
        $rules[] = [
            'files',
            'required',
            'on' => self::SCENARIO_UPLOAD
        ];
        $rules[] = [
            'files',
            'file',
            'maxFiles' => 10, // Configuratable?
            // 4 mb pro datei
            // 100 mb max pro user
            'on' => self::SCENARIO_UPLOAD
        ];
        return $rules;
    }

    /**
     * Create a new directory after validating
     *
     * @throws InvalidConfigException
     * @return bool
     */
    public function upload(): bool
    {
        if ($this->validate()) {
            $success = true;
            $uploadedFiles = [];
            $path = $this->basePath ? $this->basePath . DIRECTORY_SEPARATOR : '';
            foreach ($this->files as $file) {
                $filename = Inflector::slug(pathinfo($file,  PATHINFO_BASENAME)) . '-upload-' . time() . '.' . pathinfo($file, PATHINFO_EXTENSION);
                $filepath = $path . $filename;

                $stream = fopen($file->tempName, 'rb+');
                if ($this->getFileStorage()->writeStream($filepath, $stream) === false) {
                    $uploaded = false;
                    $success = false;
                    // break here?
                } else {
                    $uploaded = StorageItemHelper::upsertFileInDb($path, $filename);
                }
                $uploadedFiles[] = [
                    'success' => $uploaded,
                    'name' => $filename,
                    'path' => $path,
                    'message' => ''
                ];
                if (is_resource($stream)) {
                    fclose($stream);
                }
            }
            $this->setInfos($uploadedFiles);
            if ($success) {
                return true;
            }
        }
        return false;
    }

    /**
     * @inheritdoc
     */
    public function getResponse(): array
    {
        $action = false;

        if ($this->getScenario() === self::SCENARIO_UPLOAD) {
            $action = $this->upload();
        }

        return [
            'success' => $action,
            'infos' => $this->getInfos(),
            'message' => array_values($this->getFirstErrors())[0] ?? ''
        ];
    }

    /**
     * @return array
     */
    public function getInfos(): array
    {
        return $this->_infos;
    }

    /**
     * @param array $infos
     */
    public function setInfos(array $infos): void
    {
        $this->_infos = $infos;
    }

}
