<?php
/**
 * @var View $this
*/

use yii\web\View;

?>

<div id="<?php echo $this->context->getId() ?>" class="filemanager" @click="closeContextMenu()">
    <div class="navigation-bar">
        <div class="navigation-bar-item"></div>
        <div class="navigation-bar-item">
            <div v-if="options.allowUpload" class="file-upload">
                <input type="file" @change="uploadFiles" ref="fileUploadInput" multiple autocomplete="off" required="required">
                <button @click.prevent="openFilePicker"><?php echo Yii::t('filemanager', 'Upload file')?></button>
            </div>
        </div>
    </div>

    <div class="navigation-bar">
        <div class="navigation-bar-item">
            <div v-if="options.showCategoryFilter" class="category-filter">
                <select v-model="model.selectedCategory">
                    <option value=""><?php echo Yii::t('filemanager', 'All categories')?></option>
                    <option v-for="category in plugins.category.categories" :value="category.name" :key="category.id">{{ category.name }}</option>
                </select>
            </div>
        </div>
        <div class="navigation-bar-item">
            <div v-if="options.showSearch" class="file-search">
                <input type="text" v-model="model.searchTerm" placeholder="<?php echo Yii::t('filemanager', 'Search')?>" />
            </div>
        </div>
    </div>


    <ul class="navigation" v-if="options.showBreadcrumb">
        <li><button @click.prevent="moveToDirectory('')" :disabled="navigationButtonIsDisabled('')"><?php echo Yii::t('filemanager', 'Home')?></button></li>
        <li v-for="(_path, index) in paths" :key="index"><button @click.prevent="moveToDirectory(_path.path || _path)" :disabled="navigationButtonIsDisabled(_path)">{{ _path.name || _path }}</button></li>
    </ul>

    <div class="items-table">
        <div class="items-header">
            <div class="item-header-property item-header-property-sortable" @click="sort('name')"><?php echo Yii::t('filemanager', 'Name')?></div>
            <div class="item-header-property item-header-property-sortable" @click="sort('lastModified')"><?php echo Yii::t('filemanager', 'Last Modified')?></div>
            <div class="item-header-property item-header-property-sortable" v-if="isPluginEnabled('extension')"  @click="sort('extension')"><?php echo Yii::t('filemanager', 'Type')?></div>
            <div class="item-header-property item-header-property-sortable" @click="sort('size')"><?php echo Yii::t('filemanager', 'Size')?></div>
            <div class="item-header-property item-header-property-sortable" v-if="isPluginEnabled('category')" @click="sort('category')"><?php echo Yii::t('filemanager', 'Category')?></div>
            <div class="item-header-property"><?php echo Yii::t('filemanager', 'Actions')?></div>
        </div>
        <div class="items" :class="{'is-loading': isLoading}" @drop.prevent="uploadFileDragDrop" @dragover.prevent="dragOverFile" @dragleave.prevent="dragLeaveFile" data-drop-text="<?php echo Yii::t('filemanager', 'Drop to upload file(s)')?>">
            <div class="item" v-for="(item, index) in filteredItems" :key="index" :class="'item-' + item.type" @contextmenu.prevent="openContextMenu(item, $event)">
                <div class="item-property item-property-name" @dblclick.prevent="openDirectory(item)">
                    <form v-if="plugins.alias.editUuid === item.uuid" @submit.prevent="renameItem(item)" class="rename-item filemanager-form">
                        <input type="text" v-model="plugins.alias.newName" autocomplete="off" required="required">
                        <button type="submit"><?php echo Yii::t('filemanager', 'Save')?></button>
                    </form>
                    <span v-else>{{ itemLabel(item) }}</span>
                </div>
                <div class="item-property item-property-modified">{{ item.lastModified | timestampToDate }}</div>
                <div class="item-property item-property-extension" v-if="isPluginEnabled('extension')" >{{ item.extraMetadata.extension.extension | uppercase }}</div>
                <div class="item-property item-property-size">{{ item.size | prettyBytes }}</div>
                <div class="item-property item-property-category" v-if="isPluginEnabled('category')">
                    <select @change="changeCategory(item, $event)" v-if="item.type === 'file'">
                        <option disabled :selected="item.extraMetadata.category === null"><?php echo Yii::t('filemanager', 'No Category')?></option>
                        <option v-for="category in plugins.category.categories" :value="category.id" :key="category.id" :selected="item.extraMetadata.category === category.name">{{ category.name }}</option>
                    </select>
                    <span v-else></span>
                </div>
                <div class="item-property item-property-actions">
                    <ul>
                        <li>
                            <button @click.prevent="openDirectory(item)" v-if="item.type === 'dir'"><?php echo Yii::t('filemanager', 'Open')?></button>
                            <button @click.prevent="startRenameItem(item)" v-if="item.type === 'file'"><?php echo Yii::t('filemanager', 'Rename')?></button>
                            <button @click.prevent="deleteItem(item)"><?php echo Yii::t('filemanager', 'Delete')?></button>
                        </li>
                    </ul>
                </div>
            </div>
            <div class="items-empty" v-if="items.length === 0"><?php echo Yii::t('filemanager', 'No files have been uploaded yet')?></div>
            <div class="items-empty" v-if="items.length !== 0 && filteredItems.length === 0"><?php echo Yii::t('filemanager', 'No files found')?></div>
        </div>
    </div>

    <form @submit.prevent="createDirectory" v-if="options.allowCreateDirectory" class="create-directory filemanager-form">
        <input type="text" v-model="model.directoryName" autocomplete="off" required="required">
        <button type="submit"><?php echo Yii::t('filemanager', 'Create')?></button>
    </form>

    <ul class="context-menu" v-if="contextMenu.visible" id="context-menu" :style="contextMenuStyles">
        <li v-if="contextMenu.item.type === 'dir'">
            <button @click.prevent="moveToDirectory(fullPathFromItem(contextMenu.item))"><?php echo Yii::t('filemanager', 'Open')?></button>
        </li>
        <li>
            <button @click.prevent="deleteItem(contextMenu.item)"><?php echo Yii::t('filemanager', 'Delete')?></button>
        </li>
    </ul>
</div>


